;****************************************************************************
; Ju + Te Tiny  2k BASIC-System / 4k EMR-ES 1988
; (c) Dr. Helmut Hoyer, 1987-1989
;
; reass: Johann Spannkrebs 05.2006,  Volker Pohlers 02.2007,10.2007
;        Patrick Schfer 10/2011
;****************************************************************************

	cpu z8601
	page 0
	include stddefz8.inc

ROMsize		SET 2048
;MONTYP		SET "2K_ALT"
;MONTYP		SET "2K_NEU"
MONTYP		SET "4K"

KEYBTYP		SET "JU+TE"
;KEYBTYP	SET "VPOHLERS"

SyncOn		EQU 080h	; original: P37 fr Sync, P35 fr seriell in SW
SyncOff		EQU 07Fh
;SyncOn		EQU 020h	; P35 fr Sync, P37 frei fr Nutzung mit SIO
;SyncOff	EQU 0DFh

	if MONTYP = "4K"
ES_Begin	EQU 2000h	; Routinen im zweiten EPROM des EMR-ES
ES_Load		EQU 27DFh
ES_Save		EQU 26ADh
	endif

; AS-Funktionen
hi	function x,(x>>8)&255
lo	function x, x&255


;****************************************************************************
;  06.05.2006 (js)
;
;  Speicherbelegung, 2k-System
;
;      +-----------+-----------------------------------+
;      |1k RAM FFFF| 512 Byte Graphik-                 |
;      |       FE00|          Bildwiederholspeicher    |
;      +-----------+-----------------------------------+
;      |       FDFF| 128 Byte Systemstack              |
;      |       FD80|                                   |
;      +-----------+-----------------------------------+
;      |       FD7F| 128 Byte ASCII-                   |
;      |       FD00|          Bildwiederholspeicher    |
;      +-----------+-----------------------------------+
;      |       FCFF| 128 Byte frei fr Standard-       |
;      |       FC80|          Programme                |
;      +-----------+-----------------------------------+
;      |       FC7F| 128 Byte Magnetbandpuffer         |
;      |       FC00|                                   |
;      +-----------+-----------------------------------+
;
;      +-----------+-----------------------------------+
;      |       FBFF| Anwenderbereich                   |
;      |       E000|                                   |
;      +-----------+-----------------------------------+
;
;      +-----------+-----------------------------------+
;      |2k ROM 27FF| 2048 Byte Betriebssystem          |
;      |Modul2 2000|           4K-System Teil 2        |
;      +-----------+-----------------------------------+
;
;      +-----------+-----------------------------------+
;      |2k ROM 0FFF| 2048 Byte Betriebssystem          |
;      |Modul1 0800|                                   |
;      +-----------+-----------------------------------+
;      |2k ROM 07FF| 2048 Byte Basicinterpreter        |
;      |intern 0000|                                   |
;      +-----------+-----------------------------------+
;
;
;  19.10.2011 (ps)
;
;  Registerbelegung
;
; %02-03 Arbeitsregister X
; %04-05 Arbeitsregister Y
; %06-07	BASIC-Startadresse, default %E000
; %08-09 Adresse der Prozedurtabelle. %08=00 wenn interne verwendet
; %0E		BASIC-Fehler
; 		Fehlerbits: 	0..3 GOSUB Rekursionstiefe
;                 		4  GOSUB Stackberlauf (01)
;                 		5  RETURN ohne GOSUB (02)
;                 		6 Division durch Null (04)
;                 		7 Integerberlauf (08)
; %0F		BASIC-Flags
;		02: Programm beendet
;		04:
;		08: STOP aktiv
; fr BASIC-Editor
; %10-11	Pointer auf BASIC-Programmzeile
; %12-13	temp	z.B. Wort fr Dezimalausgabe
; 			%13 Zeichen von getchar
; %15-19	temp	z.B. ASCII-Stellen fr Dezimalausgabe
; 			%15 Zeichen an putchar
; %20-21	BASIC-Variable A
; %22-23	BASIC-Variable B
;   (...)
; %4E-4F	BASIC-Variable X
; %50-51	BASIC-Variable Y oder Programmstart fr SAVE
; %52-53	BASIC-Variable Z oder Programmende fr SAVE
; fr Betriebsystem
; %54		Videozeilenzhler
; %58		temp	z.B. Textnummer
; %5A		ASCII-bergaberegister
; %5B		Cursorposition im Zeichenspeicher (0..127)
; %5C		temp	z.B. Schrittweite beim Cursor
; %5D		HexByte-bergaberegister
; %5E-5F	Adress-bergaberegister
; %60		temp
; %6C		Byte aus Hex-Eingabe ber Tastatur
; %6D		Tastencode-Register. T-Bit (bit 7) =1 wenn neuer Inhalt
; %6E-6F	Wort aus Hex-Eingabe ber Tastatur
; %74-75	IRQ0-Vektor
; %76-77	IRQ1-Vektor
; %78-79	IRQ2-Vektor
; %7A-7B	IRQ3-Vektor
; %7C-7D	IRQ4-Vektor (T0 fr Videoerzeugung)
; %7E-7F	IRQ5-Vektor
;
;
;  Speicherbelegung
;
; %FFFF		Shift-Flag. 0 wenn Shift aktiv
;               Das gibt einen weissen Strich unten rechts auf dem Bildschirm.
;
;
;  BASIC
:
; 10 PRINT "HALLO"	E000: 80 0A 50 22 48 41 4C 4C 4F 22 0D
; 20 GOTO 10		E00B: 80 14 47 31 30 0D
;			E011: 00
;
; d.h. eine Zeile beginnt mit 2 byte Zeilennr binr, oberstes bit gesetzt
; dann Befehlstoken bzw ASCII, dann %0D als Zeilenende.
; Semikolon trennt Anweisungen, Komma trennt Argumente
;
;****************************************************************************

;****************************************************************************
;
;   Skizzen  zum Verstehen der Arbeitsweise des Betriebssystems
;
; - Aufbau Textpuffer und Pixelspeicher,
;   die Ausgabe des TINY ist grundsaetzlich grafisch,
;   die Zeichenmuster (ZG) sind im ROM abgelegt (Aufbau siehe ZG_TINY),
;
;   die Verwaltung des Pixelspeichers erfolgt mit einem Zeichenpuffer,
;   dieser hat eine Groesse von 8 Zeilen zu 16 Spalten,
;   wovon nur 13 dargestellt werden,
;
;   bei Textausgaben ist also zuerst der Zeichenpuffer zu manipulieren ,
;   und im 2.Schritt mit Hilfe des ZG ein Muster im Pixelspeicher zu
;   erzeugen,
;
;   Aufbau Zeichenpuffer, Textspeicher,
;            +---+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
;            |   |0|1|2|3|4|5|6|7|8|9|A|B|C|D|E|F|
;            +---+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
;   1.Zeile  |FD0| | | | | | | | | | | | | |x|x|x|
;            +---+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
;   2.Zeile  |FD1| | | | | | | | | | | | | |x|x|x|
;            +---+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
;   3.Zeile  |FD2| | | | | | | | | | | | | |x|x|x|
;            +---+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
;   4.Zeile  |FD3| | | | | | | | | | | | | |x|x|x|
;            +---+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
;   5.Zeile  |FD4| | | | | | | | | | | | | |x|x|x|
;            +---+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
;   6.Zeile  |FD5| | | | | | | | | | | | | |x|x|x|
;            +---+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
;   7.Zeile  |FD6| | | | | | | | | | | | | |x|x|x|
;            +---+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
;   8.Zeile  |FD7| | | | | | | | | | | | | |x|x|x|
;            +---+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
;
;   Adresse %FD00 ist auch auf den Bildschirm links oben,
;   Adressen %D,%E,%F (x) werden nicht benutzt,
;
;   Aufbau Pixelspeicher
;   eine Zeile je 8 Byte,
;   bei Textausgabe werden die ersten 7 Zeilen und die letzte
;   nicht benutzt,
;
;            +----+---+---+---+---+---+---+---+---+
;   7 Zeilen |FE00|   |   |   |   |   |   |   |   |
;            + -  +---+---+---+---+---+---+---+---+
;   n.b.     |FE30|   |   |   |   |   |   |   |   |
;         ---+----+---+---+---+---+---+---+---+---+
;   1.Text-  |FE38|   |   |   |   |   |   |   |   |
;            + -  +---+---+---+---+---+---+---+---+
;     zeile  |FE68|   |   |   |   |   |   |   |   |
;         ---+----+---+---+---+---+---+---+---+---+
;   2.Text-  |FE70|   |   |   |   |   |   |   |   |
;            + -  +---+---+---+---+---+---+---+---+
;     zeile  |FEA0|   |   |   |   |   |   |   |   |
;         ---+----+---+---+---+---+---+---+---+---+
;   3.Text-  |FEA8|   |   |   |   |   |   |   |   |
;            + -  +---+---+---+---+---+---+---+---+
;     zeile  |FED8|   |   |   |   |   |   |   |   |
;         ---+----+---+---+---+---+---+---+---+---+
;   4.Text-  |FEE0|   |   |   |   |   |   |   |   |
;            + -  +---+---+---+---+---+---+---+---+
;     zeile  |FF10|   |   |   |   |   |   |   |   |
;         ---+----+---+---+---+---+---+---+---+---+
;   5.Text-  |FF18|   |   |   |   |   |   |   |   |
;            + -  +---+---+---+---+---+---+---+---+
;     zeile  |FF48|   |   |   |   |   |   |   |   |
;         ---+----+---+---+---+---+---+---+---+---+
;   6.Text-  |FF50|   |   |   |   |   |   |   |   |
;            + -  +---+---+---+---+---+---+---+---+
;     zeile  |FF80|   |   |   |   |   |   |   |   |
;         ---+----+---+---+---+---+---+---+---+---+
;   7.Text-  |FF88|   |   |   |   |   |   |   |   |
;            + -  +---+---+---+---+---+---+---+---+
;     zeile  |FFB8|   |   |   |   |   |   |   |   |
;         ---+----+---+---+---+---+---+---+---+---+
;   8.Text-  |FFC0|   |   |   |   |   |   |   |   |
;            + -  +---+---+---+---+---+---+---+---+
;     zeile  |FFF0|   |   |   |   |   |   |   |   |
;         ---+----+---+---+---+---+---+---+---+---+
;   n.b.     |FFF8|   |   |   |   |   |   |   |   |
;         ---+----+---+---+---+---+---+---+---+---+
;
;   13 Zeichen werden in 8 Byte dargestellt,
;   das bedeutet eine "schoene Schieberei" in der Zeichenroutine,
;
;   Beispiel einer Textdarstellung in allen moeglichen Positionen
;
;            +--------+--------+--------+--------+--------+--------+--------+--------+
;            |   0    |   1    |   2    |   3    |   4    |   5    |   6    |   7    |
;            +--------+--------+--------+--------+--------+--------+--------+--------+
;            |76543210|76543210|76543210|76543210|76543210|76543210|76543210|76543210|
;            +--------+--------+--------+--------+--------+--------+--------+--------+
;        FE00|........|........|........|........|........|........|........|........|
;        FE08|........|........|........|........|........|........|........|........|
;        FE10|........|........|........|........|........|........|........|........|
;        FE18|........|........|........|........|........|........|........|........|
;        FE20|........|........|........|........|........|........|........|........|
;        FE28|........|........|........|........|........|........|........|........|
;        FE30|........|........|........|........|........|........|........|........|
;        ----+--------+--------+--------+--------+--------+--------+--------+--------+
;        FE38|.##....#|..###..#|##..#...|.####..#|#..####.|.##...##|...##..#|##...###|
;        FE40|#.##..##|.....#..|..#.#.#.|.#....#.|......#.|#..#.#..|#.#..#.#|..#.#...|
;        FE48|#..#...#|...##...|##..####|.###..##|#....#..|.##...##|#.#..#.#|##..#...|
;        FE50|##.#...#|..#.....|..#...#.|....#.#.|.#..#...|#..#....|#.####.#|..#.#...|
;        FE58|.##...##|#.####.#|##....#.|.###...#|#...#...|.##...##|..#..#.#|##...###|
;        FE60|........|........|........|........|........|........|........|........|
;        FE68|........|........|........|........|........|........|........|........|
;        ----+--------+--------+--------+--------+--------+--------+--------+--------+
;        FE70|.##....#|..###..#|##..#...|.####..#|#..####.|.##...##|...##..#|##...###|
;        FE78|#.##..##|.....#..|..#.#.#.|.#....#.|......#.|#..#.#..|#.#..#.#|..#.#...|
;        FE80|#..#...#|...##...|##..####|.###..##|#....#..|.##...##|#.#..#.#|##..#...|
;        FE88|##.#...#|..#.....|..#...#.|....#.#.|.#..#...|#..#....|#.####.#|..#.#...|
;        FE90|.##...##|#.####.#|##....#.|.###...#|#...#...|.##...##|..#..#.#|##...###|
;        FE98|........|........|........|........|........|........|........|........|
;        FEA0|........|........|........|........|........|........|........|........|
;        ----+--------+--------+--------+--------+--------+--------+--------+--------+
;        FEA8|.##....#|..###..#|##..#...|.####..#|#..####.|.##...##|...##..#|##...###|
;        FEB0|#.##..##|.....#..|..#.#.#.|.#....#.|......#.|#..#.#..|#.#..#.#|..#.#...|
;        FEB8|#..#...#|...##...|##..####|.###..##|#....#..|.##...##|#.#..#.#|##..#...|
;        FEC0|##.#...#|..#.....|..#...#.|....#.#.|.#..#...|#..#....|#.####.#|..#.#...|
;        FEC8|.##...##|#.####.#|##....#.|.###...#|#...#...|.##...##|..#..#.#|##...###|
;        FED0|........|........|........|........|........|........|........|........|
;        FED8|........|........|........|........|........|........|........|........|
;        ----+--------+--------+--------+--------+--------+--------+--------+--------+
;        FEE0|.##....#|..###..#|##..#...|.####..#|#..####.|.##...##|...##..#|##...###|
;        FEE8|#.##..##|.....#..|..#.#.#.|.#....#.|......#.|#..#.#..|#.#..#.#|..#.#...|
;        FEF0|#..#...#|...##...|##..####|.###..##|#....#..|.##...##|#.#..#.#|##..#...|
;        FEF8|##.#...#|..#.....|..#...#.|....#.#.|.#..#...|#..#....|#.####.#|..#.#...|
;        FF00|.##...##|#.####.#|##....#.|.###...#|#...#...|.##...##|..#..#.#|##...###|
;        FF08|........|........|........|........|........|........|........|........|
;        FF10|........|........|........|........|........|........|........|........|
;        ----+--------+--------+--------+--------+--------+--------+--------+--------+
;        FF18|.##....#|..###..#|##..#...|.####..#|#..####.|.##...##|...##..#|##...###|
;        FF20|#.##..##|.....#..|..#.#.#.|.#....#.|......#.|#..#.#..|#.#..#.#|..#.#...|
;        FF28|#..#...#|...##...|##..####|.###..##|#....#..|.##...##|#.#..#.#|##..#...|
;        FF30|##.#...#|..#.....|..#...#.|....#.#.|.#..#...|#..#....|#.####.#|..#.#...|
;        FF38|.##...##|#.####.#|##....#.|.###...#|#...#...|.##...##|..#..#.#|##...###|
;        FF40|........|........|........|........|........|........|........|........|
;        FF48|........|........|........|........|........|........|........|........|
;        ----+--------+--------+--------+--------+--------+--------+--------+--------+
;        FF50|.##....#|..###..#|##..#...|.####..#|#..####.|.##...##|...##..#|##...###|
;        FF58|#.##..##|.....#..|..#.#.#.|.#....#.|......#.|#..#.#..|#.#..#.#|..#.#...|
;        FF60|#..#...#|...##...|##..####|.###..##|#....#..|.##...##|#.#..#.#|##..#...|
;        FF68|##.#...#|..#.....|..#...#.|....#.#.|.#..#...|#..#....|#.####.#|..#.#...|
;        FF70|.##...##|#.####.#|##....#.|.###...#|#...#...|.##...##|..#..#.#|##...###|
;        FF78|........|........|........|........|........|........|........|........|
;        FF80|........|........|........|........|........|........|........|........|
;        ----+--------+--------+--------+--------+--------+--------+--------+--------+
;        FF88|.##....#|..###..#|##..#...|.####..#|#..####.|.##...##|...##..#|##...###|
;        FF90|#.##..##|.....#..|..#.#.#.|.#....#.|......#.|#..#.#..|#.#..#.#|..#.#...|
;        FF98|#..#...#|...##...|##..####|.###..##|#....#..|.##...##|#.#..#.#|##..#...|
;        FFA0|##.#...#|..#.....|..#...#.|....#.#.|.#..#...|#..#....|#.####.#|..#.#...|
;        FFA8|.##...##|#.####.#|##....#.|.###...#|#...#...|.##...##|..#..#.#|##...###|
;        FFB0|........|........|........|........|........|........|........|........|
;        FFB8|........|........|........|........|........|........|........|........|
;        ----+--------+--------+--------+--------+--------+--------+--------+--------+
;        FFC0|.##....#|..###..#|##..#...|.####..#|#..####.|.##...##|...##..#|##...###|
;        FFC8|#.##..##|.....#..|..#.#.#.|.#....#.|......#.|#..#.#..|#.#..#.#|..#.#...|
;        FFD0|#..#...#|...##...|##..####|.###..##|#....#..|.##...##|#.#..#.#|##..#...|
;        FFD8|##.#...#|..#.....|..#...#.|....#.#.|.#..#...|#..#....|#.####.#|..#.#...|
;        FFE0|.##...##|#.####.#|##....#.|.###...#|#...#...|.##...##|..#..#.#|##...###|
;        FFE8|........|........|........|........|........|........|........|........|
;        FFF0|........|........|........|........|........|........|........|........|
;        ----+--------+--------+--------+--------+--------+--------+--------+--------+
;        FFF8|........|........|........|........|........|........|........|........|
;        ----+--------+--------+--------+--------+--------+--------+--------+--------+
;
;****************************************************************************


;-----------------------------------------------------------------------------
; Einsprungadressen
;-----------------------------------------------------------------------------
	org 0000h
	phase 0800h


	if (MONTYP = "2K_ALT") || (MONTYP = "2K_NEU")
		jp 0E000h		; IRQ 0
		jp 0E003h
		jp 0E006h
		jp 0E009h
		jp IRQ4_Handler		; IRQ 4
		jp 0E00Fh		; IRQ 5
		jp begin		; Eintritt aus Bootstrap Module

	elseif MONTYP = "4K"
		jp @74h			; IRQ 0
		nop
		jp @76h			; IRQ 1
		nop
		jp @78h			; IRQ 2
		nop
		jp @7Ah			; IRQ 3
		nop
		jp @7Ch			; IRQ 4
		nop
		jp @7Eh			; IRQ 5
		nop
		jp ES_Begin		; Eintritt aus Bootstrap Module, Sprung ins 4K2-ROM
	endif

; Einsprungziele fr den U883
get_char:	jp get_char1

put_char:	ld 5Ah, 15h		; Ausgabe des in %5A bergebenen ASCII auf dem Bildschirm
		jr ZBS			;   mit %5B als Pointer

get_char1:	call TBS		; statische Tastaturabfrage mit	Ausfhrung auf Bildschirm
		ld 13h, 5Ah		;   %5B: Bildschirmpointer, %5A: ASCII
		ret


;-----------------------------------------------------------------------------
; TBS: statische Tastaturabfrage mit Echo auf Bildschirm
; %5B: Bildschirmpointer, %5A: ASCII
;-----------------------------------------------------------------------------

TBS:		call TSK		; statische Tastenabfrage. %6D:	T-Bit, %5A: ASCII

;-----------------------------------------------------------------------------
; ZBS: Ausgabe des in %5A bergebenen ASCII auf dem Bildschirm mit %5B als
; Pointer, Verarbeitung der Cursortasten, CR, CLR, CLS
;-----------------------------------------------------------------------------

ZBS:		cp 5Ah, #7Fh		; Taste "OFF" gedrckt?
		jr Z, ZBS2		;  --> Eingabe abbrechen!

		cp 5Ah, #20h		; druckbares Zeichen?
		jr NC, ZBS4		;  --> ausgeben!

		cp 5Ah, #1Bh		; Taste "Cursor rechts" ?
		jr Z, INK		;  --> Pointer erhhen, ggf. Zeilenschaltung und Bildrollen

		ld 5Ch, #-10h
		cp 5Ah, #1Ah		; Taste "Cursor hoch" ?
		jr Z, EDK		;  --> Pointer um Wert in %5C (-16) erhhen

		ld 5Ch, #10h
		cp 5Ah, #0Ah		; Taste "Cursor runter" ?
		jr Z, EDK		;  --> Pointer um Wert in %5C (+16) erhhen

		ld 5Ch, #-1
		cp 5Ah, #0Bh		; Taste "Cursor links" ?
		jr Z, EDK		;  --> Pointer um Wert in %5C (+1) erhhen

		cp 5Ah, #8		; Taste "CLR" ?
		jr NZ, ZBS1		;  nein --> weiter
		call EDK		;  --> Pointer um Wert in %5C (+1) erhhen
		ld 5Ah, #20h		; Leerzeichen
		call PrtChar		; in Bildschirmspeicher eintragen
		ld 5Ah, #8
		ret

ZBS1:		cp 5Ah, #0Dh		; Taste "ENTER" ?
		jr NZ, ZBS3		; nein --> weiter
		call NLN		; neue Bildschirmzeile mit Freizeilensperre
		ld 5Ah, #0Dh
ZBS2:		ret

ZBS3:		cp 5Ah, #0Ch		; Taste "CLS" ?
		jr Z, CLS		;  --> Lschen von Bildschirm und Pointer %5B
		ret


;-----------------------------------------------------------------------------
; ZBS4: Darstellen des ASCII aus %5A auf dem Bildschirm, Kursor (%5B) wird
; aktualisiert. Steuerzeichen werden nicht erkannt
;-----------------------------------------------------------------------------

ZBS4:		call PrtChar

;-----------------------------------------------------------------------------
; INK: Erhhen des Bildschirmpointers %5B um einen Schritt
; EDK: Erhhen des Bildschirmpointers %5B um mehrere Schritte, Anzahl in %5C.
; (incl. Zeilenschaltung und Bildschirmrollen)
;-----------------------------------------------------------------------------

INK:		ld 5Ch, #1		; Schrittweite auf +1 setzen

EDK:		add 5Bh, 5Ch		; Pointer um Schrittweite erhhen
		jr OV, EDK2		; >128? --> scrollen
		jr MI, CLS4		; =128? --> auf Null setzen

		ld 60h, 5Ch		; Schrittweite sichern
		ld 5Ch, 5Bh		; Pointer kopieren
		and 5Ch, #0Fh		; Position in der Zeile ausmaskieren
		cp 5Ch, #0Dh		; <13 ?
		jr C, EDK7		; ja --> fertig

		rl 60h
		jr NC, EDK1		; Schrittweite < 128 ?
		sub 5Bh, #6

EDK1:		add 5Bh, #3		; 3 dabei fr neue Zeile
		jr NOV, EDK7		;  und fertig

EDK2:		push RP
		srp #60h		; klarmachen zum Scrollen
	assume RP:060h
		ld R0, #0FDh		; erste Zeile Zeichenspeicher
		ld R1, #000h
		ld R2, #0FDh		; zweite Zeile
		ld R3, #010h
		ld R4, #070h		; 7x 16(13) Zeichen
EDK3:		lde R5, @RR2
		lde @RR0, R5
		inc R1
		inc R3
		djnz R4, EDK3

		ld R4, #10h		; 16 Zeichen der untersten Zeile
		ld R5, #20h		;  mit Leerzeichen
EDK4:		lde @RR0, R5		;  fllen
		inc R1
		djnz R4, EDK4

		ld R1, #38h		; erste Zeile Pixelspeicher
		inc R2
		inc R0
		ld R3, #70h		; zweite Zeile
		ld 5Bh, R3
EDK5:		lde R5, @RR2
		lde @RR0, R5
		incw RR0
		incw RR2
		cp R3, #0F0h
		jr C, EDK5
		ld R5, #0FFh
		cp R2, R5
		jr C, EDK5		; bis unteres Ende (%FFF0) erreicht

		ld R4, #38h
EDK6:		lde @RR0, R5		; die nchsten 56 bytes mit %FF fllen
		inc R1
		djnz R4, EDK6

		pop  RP			; und fertig
EDK7:		ret


;-----------------------------------------------------------------------------
; Lschen von Bildschirm und Pointer %5B
;-----------------------------------------------------------------------------

CLS:		push  RP
		srp #60h
	assume RP:060h
		ld R4, #0FDh		; Ende des Zeichenspeichers
		ld R5, #80h
		ld R3, #20h		; Leerzeichen
		ld R2, #128		; Lnge des Zeichenspeichers
CLS1:		dec R5
		lde @RR4, R3		; Zeichenspeicher lschen
		djnz R2, CLS1

		ld R2, #2
		ld R3, #0FFh		; schwarze Pixel
CLS2:		inc R4			; das gibt %FE00, Beginn des Pixelspeichers
CLS3:		lde @RR4, R3		; Pixelspeicher lschen
		djnz R5, CLS3
		djnz R2, CLS2
		pop RP

CLS4:		clr 5Bh			; Cursor auf Home-Position oben links
		ret


;-----------------------------------------------------------------------------
; Eintritt aus Bootstrap Module
;-----------------------------------------------------------------------------

begin:		srp #0F0h		; SFR-Page
	assume RP:0F0h
		ld SPL, #0		; Stackpointer auf %FE00
		ld SPH, #0FEh
		ld IMR, #10h		; IRQ4 erlauben, INTs aus
		ld IPR, #8		; IRQ4 auf niedrigste Prioritt
		ld P01M, #92h		; P0/P1 Adressbus, Stack extern, Timing normal
	if	(MONTYP	= "2K_ALT") || (MONTYP = "2K_NEU")
		ld PRE0, #5		; Prescaler =1, Timer modulo N
	elseif	MONTYP = "4K"
		ld PRE0, #7		; hier setzen wir das reservierte bit1 =1
	endif
		ld T0, #64		; Timer 0 auf 64 (d.h. INT4 alle 64s bzw 15,625kHz)
		ld TMR, #3		; Timer 0 laden und starten
		ld 06h,	#0E0h
		clr 07h			; BASIC-Programmstart auf %E000
		clr 08h
	if (MONTYP = "2K_ALT") || (MONTYP = "2K_NEU")
		clr 6Dh			; Tastaturpuffer lschen
	elseif MONTYP = "4K"
		clr P3M			; Port 2 open drain, Port 3 I/O, Parity off
	endif
		call CLS		; Lschen von Bildschirm und Pointer %5B
		ei			; Interrupts einschalten
		ld 58h, #14h		; Textnummer von "COMP JU+TE R"
		call PrtText		; Text ausgeben

; Hauptschleife des BASIC-Editors
BasCmdLoop:	srp #10h
	assume RP:010h
		ld 5Ah, #'K'
		call PrtChar		; Cursorbuchstaben drucken
		call TBS		; auf Eingabe warten

		ld R6, 5Ah		; geholtes Zeichen
		ld R15, #16h		; Zeiger auf R6
		call 03D9h		; U883BAS: Test auf Buchstabe 'A'..'Z'
		jr NC, BCL_EdCmd	;  nein --> weiter

; Buchstabe: Variablenwert ausgeben
		sub R6, #31h		; d.h. %10 fr A, %2B fr Z
		add R6, R6		; mal 2
		ld R2, @R6		; MSB in %12
		inc R6
		ld R3, @R6		; LSB in %13
		ld 5Ah, #'='
		call ZBS4		; Gleichzeichen drucken
		call PrtDezNr		; Variablenwert + NLN ausgeben
		jr BasCmdLoop		; und zurck zur Eingabeschleife

; Sonderzeichen: Editor-Kommandos
BCL_EdCmd:	cp R6, #'*'		; CONT ?
		jr NZ, BCL_NotCont	;  nein --> weiter
		tm 0Fh,	#8		; sind wir auf STOP?
		jr Z, PrtError1		;  nein --> Fehler 0
		and 0Fh,#0F7h		; STOP-Flag lschen
		jr BCL_Run1		; und Programm fortsetzen

BCL_NotCont:	ld 0Fh,#4
		clr 0Eh			; Fehlerflags lschen
		ld R0, 06h		; Pointer auf Anfang des BASIC-Programmes
		ld R1, 07h

		cp R6, #8		; NEW ?
		jr NZ, BCL_NotNew	;  nein --> weiter
		ld R4, #0
		lde @RR0, R4		; erstes Byte des BASIC-Programmes auf 0 setzen
		jr BasCmdLoop		; und zurck zur Eingabeschleife

BCL_NotNew:	cp R6, #'+'		; RUN ?
		jr NZ, BCL_NotRun	;  nein --> weiter
		call 06C9h		; Variablen lschen und BASIC initialisieren

BCL_Run1:	call 0738h		; Programm fortsetzen
		call NLN		; neue Zeile
		ld 58h, #0Ch		; Textnummer von "END"
		tm 0Fh,	#2		; sind wir fertig?
		jr NZ, BCL_End		;  ok --> weiter
		dec 58h			; Textnummer von "STOP"
		tm 0Fh,	#8		; sind wir auf STOP?
		jr NZ, BCL_End		;  ok --> weiter

PrtError1:	call PrtError		; Fehler melden
BasCmdLp1:	jr BasCmdLoop		; und zurck zur Eingabeschleife

BCL_End:	call PrtText		; Text ausgeben
		push R0			; BASIC-Zeilenpointer sichern
		push R1
		decw RR0
		call 0500h
		ld R2, R4
		ld R3, R5
		call PrtDezNr		; Zeilennummer ausgeben
		pop R1
		pop R0
		jr BasCmdLp1		; und zurck zur Eingabeschleife

BCL_NotRun:	cp R6, #'-'		; LIST ?
		ld 6Eh, #0Ch
		jr NZ, BCL_NotList	;  nein --> weiter
		ld 6Fh, #0E6h

BCL_List1:	lde R2, @RR0		; MSB Zeilennummer holen
		or R2, R2		; Null?
		jr Z, BasCmdLp1		;  --> zurck zur Eingabeschleife
		incw RR0
		lde R3, @RR0		; LSB Zeilennummer holen
		and R2, #7Fh		; bit 7 im MSB lschen
		call PrtDezNr1		; und Nummer drucken
		call PrtList		; BASIC-ZEile drucken
		jr NC, PrtError1	; Fehler ?
		clr 6Dh			; Tastaturflag lschen
		call TSK		; und auf Tastendruck warten
		cp 5Ah, #"-"		; wieder LIST?
		jr NZ, BasCmdLp1	; nein, zurck zur Eingabeschleife
		incw RR0
		jr BCL_List1		; sonst nchste Zeile ausgeben

; Zahlen: BASIC-Zeile eingeben
BCL_NotList:	call 03E7h		; U883BAS: Test auf Ziffer '0'..'9'
		jr NC, BCL_LoadSave	;  C=0, keine Ziffer --> weiter
		ld R3, R6		; Ziffer als erstes Eingabezeichen in %13
		call 02F4h		;  und den Rest ber U883 p_input3 holen
		ld R15, #16h
		ld R8, #80h		; Zeilennr hat bit 15 gesetzt...
		or R8, R4
		ld R9, R5		;  ... und steht nun in R8-R9
		or R9, R5		; LSB Null? Darf nicht wg. EOF.
		jr NZ, BCL_BASIC1	;  nein --> weiter
		inc R9			;  sonst Zeilennummer um eins erhhen

BCL_BASIC1:	lde R2, @RR0		; MSB der aktuellen BASIC-Zeile holen
		or R2, R2		; Null?
		jr Z, loc_A13		;  ja -->
		and R2, #7Fh		; bit 15 ausblenden
		incw RR0
		lde R3, @RR0		; LSB holen
		call 0141h		; U883BAS: sub_141
		tm 0Fh,	#40h
		jr NZ, loc_A33
		tm 0Fh,	#20h
		jr NZ, loc_A11
		call 0593h		; U883BAS: sub_593
		incw RR0
		jr BCL_BASIC1

loc_A11:	decw RR0

loc_A13:	ld 6Fh, #0D7h
		ld R6, R8
		call sub_CA4
		ld R6, R9
		call PrtList
		jr C, loc_A2D
		call sub_A83
		inc R6
		jr MI, BasCmdLp2
		call PrtError
		jr BasCmdLp2

loc_A2D:	call sub_CA4

BasCmdLp2:	jp BasCmdLoop		; und zurck zur Eingabeschleife

loc_A33:	call sub_A83
		jr loc_A13


	if (MONTYP = "2K_ALT") || (MONTYP = "2K_NEU")

; LOAD und SAVE-Routinen ber SIO. Der Inhalt des BASIC-Speichers wird als
; SIO-Datenstrom rausgeschrieben bzw der empfangene Datenstrom in den Speicher
; geschrieben. Ein Nullbyte beendet das Ganze
;
; bit rate = xtal frequency / (128 x (PRE0/4) x T0)
;          = 8 000 000 / (128 x 8 x 13) = 601 bps

	if MONTYP == "2K_ALT"
		nop
	elseif	MONTYP	== "2K_NEU"
	endif

BCL_LoadSave:	di
		ld PRE0, #21h		; T0 prescaler 8, Zhler mod n
		ld T0, #13		; Zhlwert fr 600 bps
		ld TMR, #43h		; T0 laden und starten, Ausgang aktiv
		ld P3M, #48h		; P34 als /DM, P30 und P37 fr SIO
		ld SIO, #0FFh
		clr IRQ			; aktive Interrupts lschen

		cp R6, #5Fh		; LOAD (shift-O) ?
		jr NZ, BCL_NotLoad	;  nein --> weiter

BCL_Load1:	tm irq, #8		; auf SIO-Interrupt warten
		jr Z, BCL_Load1

		clr irq			; Interrupt lschen
		ld R6, SIO		; Daten holen
		lde @RR0, R6		; und in den BASIC-Speicher schreiben
		incw RR0		; Pointer erhhen
		or R6, R6		; 0 = Programmende?
		jr NZ, BCL_Load1	; nein, weitermachen

BCL_Exit:	jp 0020h		; Abgang ber UB883BAS Init-Routine

BCL_NotLoad:	cp R6, #40h		; SAVE (shift-P) ?
		jr NZ, BCL_Exit		;  nein --> ins BASIC

		ld R2, #14h
		call 06AAh		; etwas warten
	if MONTYP = "2K_ALT"
		inc 0E6h		; inc R6 ausgeschrieben
	elseif (MONTYP = "2K_NEU") || (MONTYP = "4K")
		inc R6			; inc R6 in einem Byte
	endif

BCL_Save1:	tm irq, #10h		; auf T0-Interrupt warten
		jr Z, BCL_Save1

		clr irq			; Interrupt lschen
		or R6, R6		;
		jr Z, BCL_Exit		; 0 = Programmende?
		lde R6, @RR0		; neues Programmbyte holen
		ld sio, R6		; und rausschicken
		incw RR0		; Pointer erhhen
		jr BCL_Save1		; und weitermachen


	elseif	MONTYP = "4K"

; LOAD- und SAVE-Routinen im KC85-Format
BCL_LoadSave:   cp R6, #5Fh		; LOAD (shift-O) ?
                jr NZ, BCL_NotLoad	;  nein --> weiter
                di
                call ES_Load		; neue LOAD-Routine aufrufen
                ei
                jr BasCmdLp2		; und zurck zur Eingabeschleife

BCL_NotLoad:    cp R6, #40h		; SAVE (shift-P) ?
                jr NZ, BasCmdLp2	;  nein, zurck zur Eingabeschleife

                ld 50h, R1		; Pointer auf Programmstart
                ld 51h, R0
BCL_Save1:	lde R6, @RR0		; Programmbyte holen
                incw RR0
                or R6, R6		; 0 = Programmende gefunden?
                jr NZ, BCL_Save1	; nein, weiter

                ld 52h, R1		; Pointer auf Programmende
                ld 53h, R0
                call CLS
                call ES_Save		; neue SAVE-Routine aufrufen
                ei
                jr BasCmdLp2		; und zurck zur Eingabeschleife


; Ende der Laderoutine aus dem oberen ROM. Ggf Ausgabe des Programmnamens auf dem
; Bildschirm, dann Rcksprung ber TBS

RB_PrintFile:	incw SPH		; eine Rcksprungadresse entfernen
                incw SPH
                inc 5Dh			; Blocknummer erhhen
                jr NZ, RB_PF1		; nicht 0 --> Lesefehler, Ende

                ld R0, #0FCh		; Pufferbeginn
                ld R1, #0
                ld R2, #12		; Textlnge
                ld R3, #0Dh		; zuerst ein Return
RB_PF2:		ld 5Ah, R3
                call ZBS		; ...ausgeben
                lde R3, @RR0		; Zeichen aus Puffer holen
                incw RR0
                djnz R2, RB_PF2		; bis Text komplett

RB_PF1:		ei			; Bildschirm wieder einschalten
                jp TBS

	endif


sub_A83:	call 0500h		; U883BAS: sub_500
		ld R2, R0
		ld R3, R1
		ld R4, R0
		ld R5, R1
	if (MONTYP = "2K_NEU") || (MONTYP = "4K")
		incw RR4
	endif

loc_A8E:	incw RR4
		lde R7, @RR4
		rl R7
		jr UGT, loc_A8E

loc_A96:	lde R7, @RR4
		lde @RR2, R7
		incw RR2
		incw RR4
		or R7, R7
		jr NZ, loc_A96
		ret


;-----------------------------------------------------------------------------
; BASIC-Editor
; Dezimalausgabe aus %12-13 ohne fhrende Nullen (RP=%10 !)
;-----------------------------------------------------------------------------

PrtDezNr1:	call 0182h		; Word %12-13 in ASCII %15-19 + Vorzeichen in R4
		cp R4, #20h		; Vorzeichen ist Leerzeichen?
		jr Z, PrtDezNr11	;   nein --> weiter
		call PrtDezNr12		; sonst Vorzeichen drucken
PrtDezNr11:	ld R11, #15h		; erste Ziffer steht in %15

PrtDezNr14:	ld R4, @R11		; Ziffer holen
		cp R4, #30h		; ist es Null?
		jr NZ, PrtDezNr13	;  nein --> zum Drucken bergehen
		inc R11			;  sonst berspringen
		cp R11, #1Ah
		jr C, PrtDezNr14

PrtDezNr12:	ld 5Ah, R4		; R4 in ASCII-bergaberegister
		jp ZBS4			;  und ausgeben

PrtDezNr13:	call PrtDezNr12		; Ziffer drucken
		inc R11			; und die nchste holen
		ld R4, @R11
		cp R11, #1Ah
		jr C, PrtDezNr13
		ret


;-----------------------------------------------------------------------------
; NLE: Zeilenschaltung
; NLN: neue Bildschirmzeile mit Freizeilensperre, d.h. nur wenn Cursor nicht
;      am Zeilenanfang steht
;-----------------------------------------------------------------------------

NLE:		ld 5Ah, #20h		; Leerzeichen in ASCII-bergaberegister
		call ZBS4		; und drucken

NLN:		tm 5Bh, #0Fh		; sind wir am Zeilenanfang?
		jr NZ, NLE		; nein --> Zeilenschaltung
		ret



;****************************************************************************
;
;   Timing:
;   Darstellung der verfuegbaren Rechenzeit
;
;            rechter          linker
;            Bildrand         Bildrand
;          +--------------+-+--------------+--------------------------------+
;          |              |H|              |                                |
;          |              |H|              |  32s sichtbare Zeile          | 192 Zeilen
;          |              |H|              |                                | durchgaengig
;          |              |H|              |                                | ohne Interrupt
;          |              |H|              |                                |
;          |              |H|              |                                |
;          |              |H|              |                                |
;          |              |H|              |                                |
;          |              |H|              |                                |
;          |              |H|              |                                |
;          |              |H|              |                                |
;          |              |H|              |                                |
;          |              |H|              |                                |
;          |              |H|              |                                |
;          |              |H|              |                                |
;          |              |H|              |                                |
;          +--------------+-+--------------++-------------------------------+
;          |              |H|               |xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx| 1 Zeile zum
;          |              |H|               |xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx| umprogrammieren,
; unterer  |              |H|               |xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx|
; Bildrand |              |H|               |xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx| 120 Zeilen
;          |              |H|               |xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx| mit Interrupt,
;          |              |H|               |xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx|
;          + - - - - - - -+-+ - - - - - - - |xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx| davon
; V-Impuls |              |V|               |xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx| 5 Zeilen
;          + - - - - - - -+-+ - - - - - - - |xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx| mit V-Impuls,
; oberer   |              |H|               |xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx|
; Bildrand +--------------+-+---------------+-------------------------------+
;                                                                             zusammen:
;                                                                             313 Zeilen * 64s
;                                                                             = 20,032ms
;
;
; H -> Horizontalsynchronimpuls
;
; V -> Vertikalsynchronimpuls, der V-Impuls wird durch invertierte H-Impulse unterbrochen,
;
; x -> freie Rechenzeit, ca. 20% der Gesamtrechenleistung koennen genutzt werden,
;
;                   1         2        3                      4
;                            +-+
;                            | |
;          +-----------------+ +---------------+-------------------------------+
;
;****************************************************************************

IRQ4_Handler:	push RP
		srp #50h
	assume RP:050h
		djnz R4, VideoInt	; wenn (%54)>0, dann sind wir noch im Strahlrcklauf

; Beginn des sichtbaren Bereiches (3x 64 = 192 Zeilen)
; Hier braucht die Bilderzeugung 100% Rechenleistung !
		ld R6, SPH		; Stackpointer sichern
		ld R7, SPL
		ld SPH, #0FEh		; Stackpointer auf Beginn des Pixelspeichers (%FE00)
		clr SPL
		ld R4, #64		; Zeilenzhler (%54) auf 64
Video4:		swap 80h		;  8   , %80 nibble tauschen, zeit totschlagen ????

; 2. HSync-Impuls
		or P3, #SyncOn		; 10   , P37 setzen  _
		and P3, #SyncOff	; 10   , P37 lschen, Horizontalsynchronimpuls

; 3. Schwarzschulter
		tm 6Dh, #80h		; 10   , Ist das T-Bit gesetzt
		jr Z, Video1		; 12/10,  nein --> kein Tasten-Piep
		xor P3, #40h		; 10   ,  sonst P36 toggeln
		jr Video2		; 12

Video1:		and P3, #SyncOff-40h	; 10   , P36 lschen (P37 auch wre aber nicht ntig)
		jr F,$			; 10   , zeitanpassung
Video2		jr F,$			; 10   , zeitanpassung

; 4. sichtbarer Teil, 8 Byte darstellen (insg. = 128 Zyklen = 32s)
		nop			;  6
		pop 80h         	; 10   , laden 1.Byte
		nop                     ;  6
		pop 80h         	; 10   , laden 2.Byte
		nop                     ;  6   ,
		pop 80h         	; 10   , laden 3.Byte
		nop                     ;  6
		pop 80h         	; 10   , laden 4.Byte
		nop                     ;  6
		pop 80h         	; 10   , laden 5.Byte
		nop                     ;  6
		pop 80h         	; 10   , laden 6.Byte
		nop                     ;  6
		pop 80h         	; 10   , laden 7.Byte
		nop                     ;  6
		pop 80h         	; 10   , laden 8.Byte

; 1. rechter Bildrand
		inc R5			;  6   , Zeile noch mal darstellen ?
		jr Z, Video3		; 12/10,  nein --> weiter
		sub SPL, #8		; 10   ,  sonst Zeiger zurueck auf Anfang der Zeile
		sbc SPH, #0		; 10
		jr Video4		; 12   ,  und das Ganze wiederholen

Video3:		ld R5, #-3		; jede Zeile wird 3x ausgeben
		nop
		nop
		djnz R4, Video4		; Zhler dekrementieren und weiter mit nchster Zeile

; erste Zeile vom unteren Bildrand
		and irq, #0EFh		; IRQ4 lschen
		or P3, #SyncOn		; HSync ein
		and P3, #SyncOff	; HSync aus
		ld SPH, R6
		ld SPL, R7		; Stackpointer wiederherstellen
		ld R4, #120		; 120 Zeilen fr Retrace werden im Interrupt erzeugt
		jr Video5


; Bilderzeugung im Interrupt whrend des Strahlrcklaufes (120 Zeilen)
VideoInt:	cp R4, #32
		jr C, VideoInt1		; die letzten 32 Zeilen sind leer
		cp R4, #37
		jr NC, VideoInt2	; die ersten 88 Zeilen ebenfalls
		and P3, #SyncOff	; dazwischen VSync mit
		or P3, #SyncOn		;  invertiertem HSync
		jr Video5

VideoInt1:	incw 56h		; Zeit schinden
		decw 56h

VideoInt2:	or P3, #SyncOn		; HSync ein
		and P3, #SyncOff	; HSync aus

Video5:		pop RP
		iret


; Position des Zeichens in R2 in der Zeichentabelle berechnen
; und in RR0 zurckliefern
	assume RP:010h
GetCharPtr:	ld R1, R2		; ASCII mal drei
		add R1, R2		; plus Tabellenanfang
		add R1, R2
		add R1, #lo(charset)-060h	; %20 ist erstes Zeichen
		ld R0, #hi(charset)
		ret


; Position %5B im Bildschirmspeicher berechnen, in RR4 zurckliefern
GetScrnPtr:	ld R5, 5Bh		; Cursorposition
		and R5, #0Fh		; Spalte innerhalb der Zeile (0..12)
		ld R3, R5
		rl R5
		rl R5
		add R3, R5		; x5  (0..60 bzw %00..3C)
		ld R5, R3
		and R3, #007h		; bit 0..2 isolieren
		and R5, #038h		; bit 3..5 isolieren
		rl R5
		swap R5
		ld R6, 5Bh		; nochmal Cursorposition
		swap R6
		ld R4, #0FEh
		and R6, #007h		; Zeile
		inc R6			; +1

GetScrPtr1:	add R5, #38h
		adc R4, #0
		djnz R6, GetScrPtr1
		ret


;-----------------------------------------------------------------------------
; Darstellen eines ASCII aus %5A auf dem Bildschirm, Kursor bleibt unverndert,
; Steuerzeichen werden nicht erkannt
;-----------------------------------------------------------------------------

PrtChar:	push RP
	assume RP:060h
		srp #60h
		ld R0, #0FDh
		ld R1, 5Bh		; Position im Zeichenspeicher
		ld R2, 5Ah		; auszugebendes Zeichen
		lde @RR0, R2		; und eintragen
		call GetCharPtr		; Zeiger auf charset berechnen (in RR0)
		ld R2, #3		; Zeichen besteht aus 3 Byte fr 6 Zeilen
		call GetScrnPtr		; Position im Bildschirmspeicher (in RR4)

PrtChar1:	ldc R6, @RR0		; obere Zeile
		call PutScrnChar	;  in Bildschirmspeicher eintragen
		ldc R6, @RR0
		swap R6			; untere Zeile
		call PutScrnChar	;  in Bildschirmspeicher eintragen
		inc R1
		djnz R2, PrtChar1	; nchstes Byte
		pop RP
		ret


; Zeichen im oberen Nibble von R6 in den Bildschirmspeicher an der Adresse
; RR4 eintragen. RR4 wird auf die nchste Zeile erhht
PutScrnChar:	lde R7, @RR4
		incw RR4
		lde R8, @RR4
		or R3, R3
		jr Z, PutScrChr1
		ld R10, R3

PutScrChr2:	rl R8			; das ist die wilde Schieberei...
		rlc R7
		rlc R9
		djnz R10, PutScrChr2

PutScrChr1:	and R7, #00Fh
		and R6, #0F0h		; und das unser Zeichen
		or R7, R6
		or R3, R3
		jr Z, PutScrChr3
		ld R10, R3

PutScrChr4:	rr R9
		rrc R7
		rrc R8
		djnz R10, PutScrChr4

PutScrChr3:	lde @RR4, R8
		decw RR4
		lde @RR4, R7
		add R5, #8
		adc R4, #0
		ret



;-----------------------------------------------------------------------------
; Textausgabe. Die Nummer des Textes steht in Register 58h
; Die Texttabelle ist TextTab
;-----------------------------------------------------------------------------

PrtText:	push RP
		srp #50h
	assume RP:050h
		ld R14, #hi(TextTab)
		ld R15, #lo(TextTab)
PrtTxt1:	ld R9, #5Ah
		ldci @R9, @RR14		; Zeichen nach %5A holen
		tm R10, #80h		; Bit 7 gesetzt?
		jr Z, PrtTxt1		;  nein, nchstes Zeichen
		dec R8			; Textnummer (%58) herunterzhlen
		jr NZ, PrtTxt1		; und weiter

; Text gefunden
		and R10, #7Fh		; bit 7 lschen
PrtTxt2:	call ZBS		; und Zeichen drucken
		ld R9, #5Ah
		ldci @R9, @RR14		; nchstes holen
		tm R10, #80h		; Bit 7 gesetzt?
		jr Z, PrtTxt2		; nein, weitermachen

		ld R10, #20h		; Leerzeichen
		pop RP
	assume RP:010h
		jp ZBS4			; ausgeben und fertig


;-----------------------------------------------------------------------------
; statische Tastenabfrage. %6D:	T-Bit, %5A: ASCII
; An der Cursorposition wird abwechselnd ein weisses Feld und das dort
; vorhandene Zeichen ausgegeben, solange bis TAS einen Tastendruck erkennt
;-----------------------------------------------------------------------------

TSK:
	IF (MONTYP = "2K_ALT") || (MONTYP = "2K_NEU")
		cp 54h, #119
	ELSEIF MONTYP = "4K"
		cp 54h, #117
	ENDIF
		jr NZ, TSK		; warte bis zum unteren Bildrand

		push RP
	assume RP:060h
		srp #60h
		ld R2, #5		; 5 Bildschirmzeilen
		call GetScrnPtr		; Cursorposition im Bildschirmspeicher

TSK_2:		ld R6, #0
		call PutScrnChar	; Strich ausgeben
		djnz R2, TSK_2		; bis Cursorblock komplett

		ld R0, #0FDh
		ld R1, 5Bh
		lde R2, @RR0		; Zeichen an Cursorposition holen
		ld 5Ah, R2
		pop RP
	assume RP:010h

TSK_1:
	if (MONTYP = "2K_ALT") || (MONTYP = "2K_NEU")
		cp 54h, #119
	elseif MONTYP = "4K"
		cp 54h, #117
	endif
		jr NZ, TSK_1		; warte bis zum unteren Bildrand

		call PrtChar		; und ausgeben
		call TAS		; dynamische Tastenabfrage aufrufen
		tm 6Dh, #7Fh		; Taste gedrckt, T-Bit gesetzt?
		jr Z, TSK		;  nein --> nochmal das Ganze!

		ld 5Ah, 6Dh		; Zeichen von TAS bernehmen
		and 6Dh, #080h		; ASCII lschen
		and 5Ah, #07Fh		; T-Bit ausmaskieren
		ret


;-----------------------------------------------------------------------------
; dynamische Tastenabfrage
;
; liefert den letzten erkannten ASCII-Code in %6D. Wenn eine Taste gedrckt
; war, ist zustzlich bit 7 (T-Bit) gesetzt.
;
; <Shift> (ASCII-Code %15) toggelt das Shift-Flag in %FFFF. Wenn das Shift-
; Flag =0 ist, wird bei dem gelesenen ASCII-Code bit 4 invertiert. Das macht
; nicht immer Sinn, erzeugt aber aus
;
;  0123456789:;<=>?
;   !"#$%&'()*+,-./
;
; und umgekehrt. %0A (Cursor hoch) wird zu %1A (Cursor runter), %0B (Cursor
; links) zu %1B (Cursor rechts).
;-----------------------------------------------------------------------------

TAS:		push RP
		srp #60h		; wir nutzen Register %60 - %62, %6D
	assume RP:060h
		ld R0, #7Fh		; Tastaturspalten auf %7F00 - %7F0F
		ld R1, #0Fh

TAS_Loop:	ldc R2, @RR0		; Spalte lesen
		and R2, #0F0h		; ungenutzte Zeilen ausblenden
		jr NZ, TAS_KeyPressd	; nicht Null --> Taste gedrckt !
		djnz R1, TAS_Loop	; weiter mit nchster Spalte

		and R13, R0		; T-Bit (bit 7) lschen
TAS_Done:	pop RP
		ret

	assume RP:010h
TAS_KeyPressd:	or R13, R13		; %6D nicht leer?
		jr NZ, TAS_Done		;  --> Abbruch!
		ld R13, #080h		; T-Bit setzen

		ld R3, #040h		; Offset fr (letzte Zeile +1)
TAS_KeyPr1:	sub R3, #10h		; eine davor
		rl R2			; Bit im Spaltenwort prfen
		jr NC, TAS_KeyPr1	; nein, diese Zeile war's nicht

		or R1, R3		; Zeilenoffset zur Spaltennummer in R1 hinzu
		ld R0, #0Fh		; die Tabelle geht von %0F00 - %0F3F
		ldc R3, @RR0		; ASCII holen
		ld R0, #0FFh
		ld R1, #0FFh
		lde R2, @RR0		; RAM-Adresse %FFFF (Shift-Flag) holen
		cp R3, #15h		; ASCII-Code %15 (Shift gedrckt)?
		jr Z, TAS_KeyPr2	;  --> Shift-Flag setzen

		or R2, R2		; Shift-Flag testen
		jr NZ, TAS_KeyPr3	; nicht null --> kein Shift
		xor R3, #10h		; ASCII-Code um 16 erhhen bzw erniedrigen
		cp R3, #1Dh		; Shift-Enter (OFF)?
		jr NZ, TAS_KeyPr3	;  nein -->
		ld R3, #7Fh		; sonst %7F zurckliefern
TAS_KeyPr3:	or R13, R3		; ASCII in %6D einbauen

		ld R2, #0		; letzte Taste war kein Shift
TAS_KeyPr2:	com R2			; Shift-Flag toggeln
		lde @RR0, R2		; und zurckschreiben
		jr TAS_Done



sub_CA4:	ld R2, R0		; Zeiger auf aktuelle BASIC-Zeile
		ld R3, R1

; Nullbyte, d.h. Programmende suchen
loc_CA8:	lde R7, @RR2		; Zeichen holen
		incw RR2
		or R7, R7		; Null, d.h. Programmende?
		jr NZ, loc_CA8		; nein --> nchstes Zeichen prfen

loc_CB0:	lde @RR2, R7		; Byte hinter Programmende auf Null setzen
		decw RR2		; Zeiger zurck auf altes Programmende
		cp R3, R1
		jr Z, loc_CC0

loc_CB8:	decw RR2
		lde R7, @RR2
		incw RR2
		jr loc_CB0

loc_CC0:	cp R0, R2
		jr C, loc_CB8
		lde @RR0, R6
		incw RR0
		ret


sub_CC9:	call sub_CA4
		jr Z, locret_CD6


sub_CCE:	call TBS		; statische Tastaturabfrage mit	Ausfhrung auf Bildschirm
		ld R6, 5Ah
loc_CD3:	cp R6, #60h
locret_CD6:	ret

		call sub_CC9
		jr Z, locret_CD6

loc_CDC:	cp R6, #20h
		jr NZ, loc_CD3
		call sub_CCE
		jr loc_CDC
		incw RR0
		lde R6, @RR0
		ld 5Ah, R6
		call ZBS		; Ausgabe des in %5A bergebenen ASCII auf dem Bildschirm mit %5B als Pointer
		or R6, R6
		jr NZ, loc_CD3
		ret



sub_CF4:	call sub_D32
		jr NC, locret_D6A
		call sub_D09
		jr NZ, loc_D56
		call @6Eh
		jr NC, locret_D6A
		call sub_D09
		jr Z, sub_D32
		jr loc_D36



sub_D09:	cp R6, #3Ch
		jr Z, locret_D6A
		cp R6, #3Dh
		jr Z, locret_D6A
		cp R6, #3Eh
		ret



sub_D17:	cp 6Fh, #0E6h
		jr Z, loc_D1F
		ld 6Fh, #0C9h

loc_D1F:	call @6Eh
		jr NC, locret_D6A
		cp R6, #22h
		jr NZ, loc_D1F
		cp 6Fh, #-1Ah
		jr Z, loc_D30
		ld 6Fh, #-29h

loc_D30:	jp @6Eh



sub_D32:	call @6Eh
		jr NC, locret_D6A

loc_D36:	call sub_D58
		jr NC, locret_D6A
		ld R10, #3
		ld R11, #0C1h
		ld R13, #3
		call 03AFh		; U883BAS: sub_3AF
		jr Z, sub_D32
		cp R6, #24h
		jr NZ, loc_D69
		call @6Eh
		jr NC, locret_D6A
		ld R13, #3
		call 03AFh		; U883BAS: sub_3AF
		jr Z, sub_D32

loc_D56:	rcf
		ret



sub_D58:	cp R6, #28h
		jr NZ, loc_D6B
		call sub_D32
		jr NC, locret_D6A
		cp R6, #29h
		jr NZ, loc_D56
		jp @6Eh

loc_D69:	scf
locret_D6A:	ret

loc_D6B:	call 03D9h		; U883BAS: Test auf Buchstabe 'A'..'Z'
		jr NC, loc_D8E
		call @6Eh
		jr NC, locret_D6A
		call 03E2h		; U883BAS: Test auf Buchstabe oder Ziffer
		jr NC, loc_D69

loc_D79:	call @6Eh
		jr NC, locret_D6A
		cp R6, #3Bh
		jr Z, loc_D69
		cp R6, #0Dh
		jr Z, loc_D69
		cp R6, #5Dh
		jr NZ, loc_D79
		jp @6Eh

loc_D8E:	ld R4, #5
		call 03E7h		; U883BAS: Test auf Ziffer '0'..'9'
		jr C, loc_DAD
		cp R6, #2Dh
		jr Z, loc_DAC
		cp R6, #25h
		jr NZ, loc_D56
loc_D9F:	call @6Eh
		jr NC, locret_D6A
		call 03F2h		; U883BAS: Test auf Hexziffer '0'..'F'
		jr NC, loc_DB8
		djnz R4, loc_D9F
		rcf
		ret

loc_DAC:	inc R4
loc_DAD:	call @6Eh
		jr NC, locret_D6A
		call 03E7h		; U883BAS: Test auf Ziffer '0'..'9'
		jr NC, loc_DB8
		djnz R4, loc_DAD
loc_DB8:	ccf
		ret

loc_DBA:	db	  2 ;
		db	27h ; '
		db	0Dh ;
		db	37h ; 7
		db	23h ; #
		db	23h ; #
		db	0Dh ;
		db	0Dh ;
		db	27h ; '
		db	0Dh ;
		db	23h ; #
		db	23h ; #
		db	3Ah ;
		db	23h ; #
		db	60h ;
		db	60h ;
		db	50h ; P
		db	0FFh ;

;-----------------------------------------------------------------------------
; BASIC-Editor
; Anzeige einer BASIC-Zeile ohne Zeilennummer ab Adresse aus %10-11,
; die Adresse wird aktualisiert
;-----------------------------------------------------------------------------

PrtList:	ld 5Ah, #'A'
		call PrtChar
		clr 58h
		call @6Eh
		jr NC, locret_DEB
		ld R2, #007h		; tab_kdo im U883
		ld R3, #0C8h
loc_DDC:	ld R13, #17h
		ldci @R13, @RR2
		inc 58h
		cp R6, R7
		jr Z, loc_DEC
		cp 58h, #12h
		jr C, loc_DDC

locret_DEB:	ret

loc_DEC:	ld R3, 58h
		add R3, #0B9h
		ld 5Ch, #-1
		call EDK		; Cursor eins zurck
		call PrtText		; Textausgabe. %58: Text-Nr.
		ld R2, #0Dh
		ld R4, #0Eh
		ldc R5, @RR2
		jp @RR4

loc_E02:	call @6Eh
		call 03D9h		; U883BAS: Test auf Buchstabe 'A'..'Z'
		jr NC, locret_DEB
		call @6Eh
		ld R6, #3Dh

loc_E0D:	call sub_D32
		jr NC, locret_DEB
		cp R6, #2Ch
		jr Z, loc_E02

loc_E17:	cp R6, #3Bh
		jr Z, PrtList
		cp R6, #0Dh
		jr Z, loc_E35

loc_E21:	rcf
		ret

loc_E23:	call @6Eh
		jr loc_E17

loc_E27:	call @6Eh
		jr NC, locret_DEB
		cp R6, #3Bh
		jr Z, PrtList
		cp R6, #0Dh
		jr NZ, loc_E27

loc_E35:	scf
		ret

loc_E37:        ld      R5, #0
                db	0Dh		; jp never
loc_E3A:        ld      R5, #1
                call    sub_CF4
                jr      NC, locret_DEB
                or      R5, R5
                jr      NZ, loc_E0D
                ld      R6, #3Bh
                ld      58h, #13h
                call    PrtText


loc_E4D:	jp	PrtList
		call	@6Eh
		cp	R6, #22h
		jr	NZ, loc_E5A
		call	sub_D17

loc_E5A:	call	3D9h		; U883BAS: Test auf Buchstabe 'A'..'Z'
		jr	C, loc_E23
		ret

loc_E60:	call	@6Eh
		cp	R6, #22h
		jr	NZ, loc_E6A
		call	sub_D17

loc_E6A:	cp	R6, #3Bh
		jr	Z, loc_E4D
		cp	R6, #0Dh
		jr	Z, loc_E35
		call	loc_D36
		jr	NC, loc_E21
		cp	R6, #2Ch
		jr	NZ, loc_E17
		jr	loc_E60


; Ausgabe "ERROR xx"

PrtError:	call NLN		; neue Zeile
		ld 58h, #12h		; Text ERROR
		call PrtText		; Textausgabe. %58: Text-Nr.
		clr R2
		ld R3, 0Eh		; Fehlercode
		and R3, #0F0h		; obere vier Byte
		swap R3			; als Zahl ausgeben

;-----------------------------------------------------------------------------
; BASIC-Editor
; Dezimalanzeige aus %12-13 ohne fhrende Nullen mit Zeilenschaltung (RP=%10 !)
;-----------------------------------------------------------------------------

PrtDezNr:	call PrtDezNr1		; Zahlenwert drucken
		jp NLN			; und neue Zeile anfangen


;-----------------------------------------------------------------------------
; Texttabelle fr Textausgabe PrtText
;-----------------------------------------------------------------------------

TextTab:	db 'L'+80h, "ET"		; #01h
		db 'P'+80h, "ROC"		; #02h
		db 'G'+80h, "OTO"		; #03h
		db 'I'+80h, "F"			; #04h
		db 'E'+80h, "LSE"		; #05h
		db 'R'+80h, "ETURN"		; #06h
		db 'G'+80h, "OSUB"		; #07h
		db 'W'+80h, "AIT"		; #08h
		db 'R'+80h, "EM"		; #09h
		db 'C'+80h, "ALL"		; #0Ah
		db 'S'+80h, "TOP"		; #0Bh
		db 'E'+80h, "ND"		; #0Ch
		db 'T'+80h, "RAP"		; #0Dh
		db 'T'+80h, "OFF"		; #0Eh
		db 'P'+80h, "RINT"		; #0Fh
		db 'P'+80h, "TH"		; #10h
		db 'I'+80h, "NPUT"		; #11h
		db 'E'+80h, "RROR"		; #12h
		db  0Bh+80h, "THEN"		; #13h
		db 'C'+80h, "OMP JU+TE R "
		db  5Ch, 5Ch, 5Ch, 5Ch, 5Ch, 5Ch, 5Ch, 5Ch, 5Ch, 5Ch, 5Ch, 5Ch
		db  0Ah				; #14h
		db 0FFh 			; Listenende

;-----------------------------------------------------------------------------
; Tastaturmatrix 16x4. Spalte 0 darf nicht belegt werden!
;-----------------------------------------------------------------------------

keytab:
	if KEYBTYP == "JU+TE"		; Original-Belegung

		db    0, 20h, 'Y', 'X', 'C', 'V', 'B', 'N', 'M', ',', '.', '/', 0Dh, 1Bh, 0, 0
		db    0, 15h, 'A', 'S', 'D', 'F', 'G', 'H', 'J', 'K', 'L', '*', '-', 0Ah, 0, 0
	if (MONTYP = "2K_ALT") || (MONTYP = "2K_NEU")
		db    0, 0Bh
	elseif MONTYP = "4K"
		db    0, '='
	endif
		db            'Q', 'W', 'E', 'R', 'T', 'Z', 'U', 'I', 'O', 'P', '+', 0Bh, 0, 0
		db    0, 0Ah, '1', '2', '3', '4', '5', '6', '7', '8', '9', '0', 08h, 1Ah, 0, 0

	elseif	KEYBTYP	== "VPOHLERS"	; Pollin-Tastatur V.Pohlers
		db    0, 0Dh, 'Y', 'X', 'C', 'V', 'M', ',',   0, 20h, 0Ah, 0Bh, '.', 'B', 'N', 1Bh
		db    0, 'L', 'A', 'S', 'D', 'F', 'J', 'K',   0,   0,   8, 1Ah, '-', 'G', 'H',   0
		db    0, 'O', 'Q', 'W', 'E', 'R', 'U', 'I',   0, 15h,   8,   0, 'P', 'T', 'Z', '+'
		db    0, '9', '1', '2', '3', '4', '7', '8',   0,   0, '/',   0, '*', '5', '6', '0'

	endif

;-----------------------------------------------------------------------------
; Zeichensatz 64 Zeichen je 4x6 Pixel, invers+normal
;-----------------------------------------------------------------------------

charset:
	include Tiny_ZG.inc

	dephase
	if $>ROMsize
		error "\aROM size exceeded !!!"
	elseif
		db ROMsize-$ dup(0FFh)	; pad with $FF's
	endif
	end


%0824	Tastenabfrage statisch mit Ausfhrung auf Bildschirm, Kursor (%5B) wird aktualisiert, %5A := ASCII
%0827	Darstellen eines ASCII aus %5A auf dem Bildschirm, Kursor (%5B) wird aktualisiert
%0872	Darstellen eines ASCII aus %5A auf dem Bildschirm, Kursor (%5B) wird aktualisiert. Steuerzeichen werden nicht erkannt
%0875	Erhhen des Kursors (%5B) um 1 mit ggf. Bildrollen
%0878	Erhhen des Kursors (%5B) um den Inhalt von %5C mit ggf. Bildrollen
%08DD	Bild lschen, Kursor (%5B) := 0
%0AA3	Dezimalausgabe aus %12 und %13 ohne fhrende Nullen RP = #%10
%0ACE	Zeilenschaltung
%0AD4	Zeilenschaltung mit Freizeilensperre
%0B95	Darstellen eines ASCII aus %5A auf dem Bildschirm, Kursor bleibt unverndert, Steuerzeichen werden nicht erkannt
%0C1D	Tastenabfrage statisch ohne Ausfhrung auf dem Bildschirm %5A:= ASCII
%0C56	Tastenabfrage dynamisch mittels Tastencoderegister (%6D): T-Bit und ASCII
%0DCC	Anzeige einer BASIC-Zeile ohne Zeilennummer ab Adresse aus %10 und %11, die Adresse wird aktualisiert RP = #%10 %6E = #%0C %6F = #%E6
%0E92	Dezimalanzeige aus %12 und %13 ohne fhrende Nullen mit Zeilenschaltung RP = #%10
%20B6	Hexadezimalanzeige eines Bytes aus %5D, Kursor (%5B) wird aktualisiert
%20CF	Hexadezimalanzeige der Speicherzelle mit der Adresse aus %5E und %5F, Adresse wird um 1 erhht, der Kursor (%5B) wird aktualisiert
%20DB	Hexadezimalanzeige eines Doppelbytes aus %5E und %5F, Kursor (%5B) wird aktualisiert
%20E6	Berechnen der Byteanzahl in %6B eines Befehls mit dem Operationscode aus %5D
%22E5	Hexadezimaleingabe eines Bytes in %6C mit Tastatur und Anzeige RP = #%60
%231F	Hexadezimaleingabe eines Doppelbytes in %6E und %6F mit Tastatur und Anzeige RP = #%60
%2352	RAM- und Registeranzeige
%23E6	Programm-Modus
%260E	Anzeige einer Maschinenprogrammzeile mit der Adresse aus %5E und %5F, Adresse und Kursor (%5B) werden aktualisiert, RP := #%60
%268F	Magnetbandausgabe RP = #%60
%27D1	Magnetbandeingabe RP = #%60
